      PROGRAM DYqT

c.....Version 1.0 (Aug 2015)

c.....Please refer to: 
c     G. Bozzi, S. Catani, G. Ferrera, D. de Florian, M. Grazzini
c     Phys. Lett. B696 (2011) 207 [arXiv:1007.2351 [hep-ph]],
c     Nucl. Phys. B815 (2009) 174 [arXiv:0812.2862 [hep-ph]];

c.....This code computes the transverse momentum (qT) distribution 
c     of Drell-Yan lepton pair  in pp/ppbar collisions at NLL+LO 
c     and NNLL+NLO accuracy.

c.....We consider vector boson (V=Z/gamma*,W+,W-) hadroproduction 
c     followed by leptonic inclusive decay (Z/gamma*->l+ l-, W->l nu_l):
c     
c             p1 p2  ->  V(q2,qT) + X  ->  l1 + l2 + X .
c
c
c     The code computes the differential cross sections:
c
c             q2*dsigma/dq2/dqT    [pb/GeV] 
c
c     where q2 (>>Lamda_QCD^2) is the squared invariant mass of 
c     the vector boson (lepton pair).
c 
c     If the "narrow width approximation" is set on true (only for 
c     Z and W production), the code computes:
c
c           dsigma_V/dqT * BR(V->l1 l2)   [pb/GeV] ,
c
c     where BR(V->l1 l2) is the leptonic branching ratio of the vector boson. 



c.....INPUT:

c.....1.Colliding hadrons (1: proton-proton, -1: proton-antiproton)
c.....2.Produced vector boson (1: photon, 2: W^+ and W^-, 21: W^+, 22: W^-,
c.....                         3: Z,  5: Z/photon)
c.....3.Narrow Width Approximation (only for W and Z prod.) (0: false, 1: true)
c.....4.Remove Branching Ratio (only within the NWA) (0: false, 1: true)
c.....5.Lepton pair invariant mass [GeV] (dummy within the NWA)
c.....6.Order of calculation (1: NLL+LO, 2: NNLL+NLO)
c.....7.Output file name
c.....8.PDF set, loop for alpha_S (for native PDFs, see pdf_nat.f for complete list of PDFs)
c.....9.PDF set name, member  (for LHAPDF interface)
c.....10.Center of mass energy [GeV]
c.....11.Renormalization, Factorization and Resummation scales [GeV]
c.....12.Non Perturbative smearing (g parameter)
c.....13.qT points (qtmin qtmax qtbin)


c.....NATIVE or LHAPDF interface can be chosen in the Makefile

c.....The output is automatically redirected to the standard output and  
c.....to a file with name defined in the input file.

c.....Loop over qT: from qtmin  to qtmax with qtbin pass


      implicit none
      integer j,ifail,flag,flag1,flag2,flag4,imod,flag5,fnwa
      integer ih1,ih2,qti,qtimax,brflag
      integer isetproton,ippbar,nloop,iord,flagrealcomplex
      real *8 v,fu,fd,max,min,tmp1,tmp2,xlam,phi
      real *8 errest,adpint,sigma0,gevpb,gf,amv,adzint
      real *8 as
      real *8 subqqb,subqg,subgg,a_param,b0p,muq
      real *8 vud,vus,vub,vcd,vcs,vcb,vtd,vts,vtb
      complex *16 S,ru,rd,rr,jac
      complex*16 alphas
      real *8 allres,expres,ris,g
      real *8 qtmin,qtmax1,qtbin1,qtmax2,qtbin2,qtmax3,qtbin3
     /,qtbin,qtmax4,qtbin4
      real *8 aweak,aem,Mw,sw2,cw2,Mz,zw,ww,tmpxx
      dimension sigmaij(-5:5,-5:5)
      real *8 sigmaij,eequ,eeqd,xmur,xmuf,xmur2,xmuf2,qtbis,ssroot
      integer si,sj,prodflag,flagch,flagfit
c.....this is for LL calculation
      complex *16 qqb1,qg1,gg1,LLqqb
      common/LLqqb/LLqqb

      common/ckm/vud,vus,vub,vcd,vcs,vcb,vtd,vts,vtb
      common/flagrealcomplex/flagrealcomplex
      common/asNEW/as
      common/v/v
      common/nloop/nloop
      common/order/iord
      common/jac/jac
      common/j/j
      common/isetproton/isetproton
      common/flag/flag
      common/flag1/flag1
      common/flag2/flag2
      common/flag4/flag4
      common/flagfit/flagfit
      common/xlam/xlam
      common/ippbar/ippbar
      common/modified/imod
      common/a_param/a_param,b0p
      common/sigma0/sigma0
      common/NP/g
!
      common/flag5/flag5
      common/prodflag/prodflag
      common/sigmaij/sigmaij
      common/gevpb/gevpb
      common/flagch/flagch
      common/sigs/siggamma,sigint,sigz,sigw
!
      external fu,fd,adpint,alphas,S,adzint
      data gevpb/3.8937966d8/
      include 'const.h'
      include 'scales.h'

      common/pdf/ih1,ih2

      real *8 y1,y2,ppi,brw,brz,siggamma,sigint,sigz,sigw

      integer ic,iter,locall,nlocall,mem,flagpdf
      common/vegint/iter,locall,nlocall
      common/cdyqt/amv,y1,y2,qtbis,gf,ppi,ssroot,sw2,aem,ic
      common/scales2/xmur,xmuf,xmur2,xmuf2
      integer*4 today(3), now(3)
      character*30 runstring
      character name*40,name2*40
      common/name/name
      common/mem/mem


c......Parameters(amv=vector boson mass, gf=gfermi)
c................(nf=number of active flavours)
c................(flag=0,1 needs to call just once the pdfs fitting routine and evolution code initialization)
c................(flag1=1,2 set the order of resummation: NLL,NNLL)
c................(ippbar=1,-1 sets p-p or p-pbar events)
c................(iord sets the order for the evolution: LO(0),NLO(1))
c................(flag2 sets the order of matching: LO(1), NLO(2))
c................(nloop sets the order of the calculation)


      read (*,*) ic                  ! collider type !

      if (ic.eq.1) then
c   pp collider
      ih1=1
      ih2=1
      elseif(ic.eq.-1) then
c   pp collider
      ih1=1
      ih2=-1
      else
      write(*,*)'ERROR: Wrong collider type !'
      stop
      endif
      ippbar=ic

      read (*,*) prodflag            ! prodflag, type of vector boson !
      flag5=prodflag

      read(*,*)fnwa                  ! flag narrow width approximation !
      read(*,*)brflag                ! brflag !


c  imod=0 original sudakov
c  imod=1 modified sudakov
      imod=1

      read (*,*) amv                      ! lepton invariant mass !

      gf=1.16637d-5
      Mz=91.1876d0                        ! z mass ! 
      Mw=80.399d0                         ! w mass !
      zw=2.4952d0                         ! z width ! 
      ww=2.085d0                          ! w width !

      nf=5
      flag=0

      if (fnwa.eq.1) then
         if (flag5.eq.3)  amv=Mz
         if (flag5.eq.2.or.flag5.eq.21.or.flag5.eq.22) amv=Mw
      endif


c     Order of the calculation: (0)-> LL  (1)-> NLL+LO (2)-> NNLL+NLO

      read (*,*) flag1

      read (*,*) runstring                !  output file name !
      open (unit=12, file=runstring, status='unknown') 

      if (flag1.eq.0) then
!         write (12,*) '(LL no matching'
      write(*,*)'ERROR: Wrong order !'
      stop
      elseif (flag1.eq.1) then
         iord=0   ! LO evolution
         flag2=1  ! LO matching      
      elseif(flag1.eq.2) then
         iord=1   ! NLO evolution
         flag2=2  ! NLO matching       
      else
      write(*,*)'ERROR: Wrong order !'
      stop
      endif

      read (*,*)isetproton,nloop

      call lambda5qcd(xlam,isetproton)
      xlam=xlam

c Initialization for LHAPFD, dummy for native PDFs
      flagpdf=0
      read (*,*)name,mem
      name2=name
      call initialPDF()

c.....Constants
c..............(CA,Cf=color factors)
c..............(beta_i=beta function coefficients defined to as/pi)
c..............(Euler=Euler's Gamma function)
      pi=dacos(-1d0)
      ppi=pi
      II=(0d0,1d0)
      ifail=0
      CA=3d0
      Cf=4/3d0
      Euler=0.57721566d0
      Z2=1.644934d0
      Z3=1.202057d0
      b0=2*dexp(-Euler)
!      b0p=b0*a_param

      beta0=(33-2*nf)/12d0
      beta1=(153-19*nf)/24d0
      beta2=2857/128d0-5033*nf/1152d0+325*nf**2/3456d0

c.....Sudakov coefficients defined 
c.....according to Catani-De Florian-Grazzini
c.....WARNING!!!
c.....they are defined according to alphas/pi normalization!!!
c.....Now I added B2-dependence from resummation scheme 
c.....both for Higgs prodution and DY process

c.....choice of resummation scheme

      flag4=1 

c.....Choosing processes and scales

      read (*,*) sroot

      ssroot=sroot
      shad=sroot**2
      q=amv
      q2=q**2
      xtau=q2/shad

c.....Renormalization and factorization scales

      read (*,*) mur,muf,muq

      a_param=q/muq   !ratio between the hard scale and resummation scale!
      b0p=b0*a_param

      mur2=mur**2
      muf2=muf**2
      xmur=mur
      xmuf=muf
      xmur2=mur2
      xmuf2=muf2
      
c.....quark coefficients
      A1q=Cf
      A2q=Cf/2d0*(67/6d0-(pi**2)/2d0-5/9d0*nf)
      A3q=Cf*(13.768339d0-2.1467277d0*nf-nf**2/108d0)            ! 
     /+Cf*(CA*(29.925926d0-28d0*Z3)-8.2962963d0*nf/2d0)  ! 
     /*2d0*(beta0*4d0)/64d0                         ! A3 qT-resummation
      B1q=-(3d0*Cf)/2d0

         B2q=Cf**2*(pi**2/4d0-3/16d0-3*Z3)+
     /        CA*Cf*(11*pi**2/36d0-193/48d0+3*Z3/2d0)+
     /        Cf*nf*(17/24d0-pi**2/18d0)
         C1qqn=Cf/2d0*(pi**2/2d0-4d0)! Only delta(1-z) part, i.e. N independent part,
                                     ! the remaining N dependent part is added in evnew.f


c.....here alphas gives alphas/4pi at the renormalization scale "mur2".
c.....if you need alphas,    you have to write as=alphas*4*pi
c.....if you need alphas/pi, you have to write as=alphas*4
c     as=alphas(cmplx(q2))*4*pi
      as=dreal(alphas(dcmplx(mur2)))*4      
!      write(*,*)'alphas(mZ)=',dreal(alphas(dcmplx(91.1876d0**2)))*4*pi


      read(*,*)g    ! Here you can choose NP smearing

      read(*,*)qtmin,qtmax1,qtbin1        ! qT points ! 
!     /,qtmax2,qtbin2,qtmax3,qtbin3,qtmax4,qtbin4 !
!      read(*,*)iter,locall,nlocall        ! vegas parameters !
      iter=15
      locall=20000
      nlocall=30000

      tmpxx=(q2/ssroot+ssroot)/dsqrt(q2)

      y2=dlog((tmpxx+dsqrt(tmpxx**2-4))/2)
      y1=-y2
      flagch=0
      flagfit=0

C  ELECTROWEAK PARAMETER SCHEME: SAME SCHEME OF DYNNLO (G_mu scheme)

c      aem=1d0/128.89d0  
c      sw2=pi*aem/(dsqrt(2d0)*gf*Mw**2)
      sw2=1d0-(Mw/Mz)**2
      cw2=1d0-sw2
      aem=dsqrt(2d0)*gf*Mw**2*sw2/pi
      aweak=aem/sw2     

      sigma0=4d0*pi**2*aweak/3d0*gevpb

      eequ=2d0/3d0
      eeqd=-1d0/3d0

      do si=-5,5 
         do sj=-5,5
            sigmaij(si,sj)=0
         enddo
      enddo

c.....Branching ratios 
!
c     Narrow Width Approximation (only for Z and W)
      if(fnwa.eq.1) then 
      if(flag5.eq.1.or.flag5.eq.4.or.flag5.eq.5) then
      write(*,*)"ERROR: Narrow Width Approximation valid only for W or Z
     / production!"
      stop 
      endif
      endif

c     BR(Z->ee)
      brz=1d0/zw*aem/24d0*Mz
     /  *((-1d0+2d0*sw2)**2+(2d0*sw2)**2)/(sw2*cw2) !=0.033638
c     brz=0.03363   !(\pm 0.00004): PDG 2010 experimental BR(Z->e,e) 
c     brz=0.03366   !(\pm 0.00007): PDG 2010 experimental BR(Z->mu,mu) 
c     BR(W->ev)
      brw=1d0/ww*aem*Mw/(12d0*sw2)  !=0.10906 
c     brw=0.1075   !(\pm 0.0009): PDG 2010 experimental BR(W->e,nu_e) 
c     brw=0.1057   !(\pm 0.0013): PDG 2010 experimental BR(W->mu,nu_mu) 
!      write(*,*) brz
 
      if (brflag.eq.1.and.fnwa.eq.0) then
      write(*,*)"ERROR: Remove BR flag can be true, only if Narrow Width
     / Approximation flag is true!"
      stop
      endif

      if (brflag.eq.1) then
            brz=1.0d0
            brw=1.0d0
      endif

      if (fnwa.eq.1) then      ! 
         sigz=brz!/(16d0*cw2)
         sigw=brw!/4d0
      elseif (fnwa.eq.0) then
         sigz=q2*brz*zw*q2/(pi*Mz)/((q2-Mz**2)**2+Mz**2*zw**2) !/(16d0*cw2)
         sigw=q2*brw*ww*q2/(pi*Mw)/((q2-Mw**2)**2+Mw**2*ww**2) !/4d0
         siggamma=q2*(aem/(3d0*pi*q2))
         sigint=-q2*aem/(6d0*pi)*(q2-Mz**2)/((q2-Mz**2)**2+Mz**2*zw**2)*
     /        (-1d0+4d0*sw2)/(2d0*sqrt(sw2*cw2))               !sqrt(sw2/cw2)/2d0
      else
      write(*,*)"ERROR: Wrong flag 'Narrow Width Approximation' !"
      stop
      endif




c.....CKM matrix element

      vud=0.97428d0  
      vus=0.2253d0
      vub=0.00347d0  
      vcd=0.2252d0   
      vcs=0.97345d0  
      vcb=0.0410d0   
!      vud=0.975d0
!      vus=0.2220d0
!      vub=0.0d0
!      vcd=0.222d0
!      vcs=0.975d0
!      vcb=0.0d0
!      vtd=0.00862d0  
!      vts=0.0403d0
!      vtb=0.999152d0
      vtd=0.0d0  
      vts=0.0d0
      vtb=0.0d0


c.....Born cross sections

      if (flag5.eq.3) then
            sigmaij(1,-1)=sigma0*(1d0+(1d0-4d0*ABS(eequ)*sw2)**2)
     \          /(16d0*cw2)*sigz
            sigmaij(4,-4)=sigmaij(1,-1)
            sigmaij(-1,1)=sigmaij(1,-1)
            sigmaij(-4,4)=sigmaij(1,-1)
c
            sigmaij(2,-2)=sigma0*(1d0+(1d0-4d0*ABS(eeqd)*sw2)**2)
     \         /(16d0*cw2)*sigz
            sigmaij(3,-3)=sigmaij(2,-2)
            sigmaij(5,-5)=sigmaij(2,-2)
            sigmaij(-2,2)=sigmaij(2,-2)
            sigmaij(-3,3)=sigmaij(2,-2)
            sigmaij(-5,5)=sigmaij(2,-2)
      elseif (flag5.eq.1) then
            sigmaij(1,-1)=sigma0*sw2*(eequ)**2*siggamma
            sigmaij(4,-4)=sigmaij(1,-1)
            sigmaij(-1,1)=sigmaij(1,-1)
            sigmaij(-4,4)=sigmaij(1,-1)
c
            sigmaij(2,-2)=sigma0*sw2*(eeqd)**2*siggamma
            sigmaij(3,-3)=sigmaij(2,-2)
            sigmaij(5,-5)=sigmaij(2,-2)
            sigmaij(-2,2)=sigmaij(2,-2)
            sigmaij(-3,3)=sigmaij(2,-2)
            sigmaij(-5,5)=sigmaij(2,-2)
      elseif (flag5.eq.4) then
            sigmaij(1,-1)=sigma0*sqrt(sw2/cw2)*ABS(eequ)/2d0*
     \(1d0-4d0*ABS(eequ)*sw2)*sigint
            sigmaij(4,-4)=sigmaij(1,-1)
            sigmaij(-1,1)=sigmaij(1,-1)
            sigmaij(-4,4)=sigmaij(1,-1)
c
            sigmaij(2,-2)=sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint
            sigmaij(3,-3)=sigmaij(2,-2)
            sigmaij(5,-5)=sigmaij(2,-2)
            sigmaij(-2,2)=sigmaij(2,-2)
            sigmaij(-3,3)=sigmaij(2,-2)
            sigmaij(-5,5)=sigmaij(2,-2)
      elseif(flag5.eq.21) then
            sigmaij(1,-2)=sigma0/4d0*sigw*vud**2
            sigmaij(-2,1)=sigmaij(1,-2)
            sigmaij(1,-3)=sigma0/4d0*sigw*vus**2
            sigmaij(-3,1)=sigmaij(1,-3)
            sigmaij(1,-5)=sigma0/4d0*sigw*vub**2
            sigmaij(-5,1)=sigmaij(1,-5)
            sigmaij(4,-3)=sigma0/4d0*sigw*vcs**2
            sigmaij(-3,4)=sigmaij(4,-3)
            sigmaij(4,-2)=sigma0/4d0*sigw*vcd**2
            sigmaij(-2,4)=sigmaij(4,-2)
            sigmaij(4,-5)=sigma0/4d0*sigw*vcb**2
            sigmaij(-5,4)=sigmaij(4,-5)
      elseif(flag5.eq.22) then
            sigmaij(-1,2)=sigma0/4d0*sigw*vud**2
            sigmaij(2,-1)=sigmaij(-1,2)
            sigmaij(-1,3)=sigma0/4d0*sigw*vus**2
            sigmaij(3,-1)=sigmaij(-1,3)
            sigmaij(-1,5)=sigma0/4d0*sigw*vub**2
            sigmaij(5,-1)=sigmaij(-1,5)
            sigmaij(-4,3)=sigma0/4d0*sigw*vcs**2
            sigmaij(3,-4)=sigmaij(-4,3)
            sigmaij(-4,2)=sigma0/4d0*sigw*vcd**2
            sigmaij(2,-4)=sigmaij(-4,2)
            sigmaij(-4,5)=sigma0/4d0*sigw*vcb**2
            sigmaij(5,-4)=sigmaij(-4,5)
      elseif(flag5.eq.2) then
            sigmaij(1,-2)=sigma0/4d0*sigw*vud**2
            sigmaij(-2,1)=sigmaij(1,-2)
            sigmaij(1,-3)=sigma0/4d0*sigw*vus**2
            sigmaij(-3,1)=sigmaij(1,-3)
            sigmaij(1,-5)=sigma0/4d0*sigw*vub**2
            sigmaij(-5,1)=sigmaij(1,-5)
            sigmaij(4,-3)=sigma0/4d0*sigw*vcs**2
            sigmaij(-3,4)=sigmaij(4,-3)
            sigmaij(4,-2)=sigma0/4d0*sigw*vcd**2
            sigmaij(-2,4)=sigmaij(4,-2)
            sigmaij(4,-5)=sigma0/4d0*sigw*vcb**2
            sigmaij(-5,4)=sigmaij(4,-5)
            sigmaij(-1,2)=sigma0/4d0*sigw*vud**2
            sigmaij(2,-1)=sigmaij(-1,2)
            sigmaij(-1,3)=sigma0/4d0*sigw*vus**2
            sigmaij(3,-1)=sigmaij(-1,3)
            sigmaij(-1,5)=sigma0/4d0*sigw*vub**2
            sigmaij(5,-1)=sigmaij(-1,5)
            sigmaij(-4,3)=sigma0/4d0*sigw*vcs**2
            sigmaij(3,-4)=sigmaij(-4,3)
            sigmaij(-4,2)=sigma0/4d0*sigw*vcd**2
            sigmaij(2,-4)=sigmaij(-4,2)
            sigmaij(-4,5)=sigma0/4d0*sigw*vcb**2
            sigmaij(5,-4)=sigmaij(-4,5)
      elseif (flag5.eq.5) then
            sigmaij(1,-1)=sigma0*(1d0+(1d0-4d0*ABS(eequ)*sw2)**2)
     \          /(16d0*cw2)*sigz
            sigmaij(4,-4)=sigmaij(1,-1)
            sigmaij(-1,1)=sigmaij(1,-1)
            sigmaij(-4,4)=sigmaij(1,-1)
c
            sigmaij(2,-2)=sigma0*(1d0+(1d0-4d0*ABS(eeqd)*sw2)**2)
     \         /(16d0*cw2)*sigz
            sigmaij(3,-3)=sigmaij(2,-2)
            sigmaij(5,-5)=sigmaij(2,-2)
            sigmaij(-2,2)=sigmaij(2,-2)
            sigmaij(-3,3)=sigmaij(2,-2)
            sigmaij(-5,5)=sigmaij(2,-2)
c
            sigmaij(1,-1)=sigmaij(1,-1)+sigma0*sw2*(eequ)**2*siggamma
            sigmaij(4,-4)=sigmaij(4,-4)+sigma0*sw2*(eequ)**2*siggamma
            sigmaij(-1,1)=sigmaij(-1,1)+sigma0*sw2*(eequ)**2*siggamma
            sigmaij(-4,4)=sigmaij(-4,4)+sigma0*sw2*(eequ)**2*siggamma
c
            sigmaij(2,-2)=sigmaij(2,-2)+sigma0*sw2*(eeqd)**2*siggamma
            sigmaij(3,-3)=sigmaij(3,-3)+sigma0*sw2*(eeqd)**2*siggamma
            sigmaij(5,-5)=sigmaij(5,-5)+sigma0*sw2*(eeqd)**2*siggamma
            sigmaij(-2,2)=sigmaij(-2,2)+sigma0*sw2*(eeqd)**2*siggamma
            sigmaij(-3,3)=sigmaij(-3,3)+sigma0*sw2*(eeqd)**2*siggamma
            sigmaij(-5,5)=sigmaij(-5,5)+sigma0*sw2*(eeqd)**2*siggamma
c
            sigmaij(1,-1)=sigmaij(1,-1)
     \+sigma0*sqrt(sw2/cw2)*ABS(eequ)/2d0*
     \(1d0-4d0*ABS(eequ)*sw2)*sigint
            sigmaij(4,-4)=sigmaij(4,-4)
     \+sigma0*sqrt(sw2/cw2)*ABS(eequ)/2d0*
     \(1d0-4d0*ABS(eequ)*sw2)*sigint
            sigmaij(-1,1)=sigmaij(-1,1)
     \+sigma0*sqrt(sw2/cw2)*ABS(eequ)/2d0*
     \(1d0-4d0*ABS(eequ)*sw2)*sigint
            sigmaij(-4,4)=sigmaij(-4,4)
     \+sigma0*sqrt(sw2/cw2)*ABS(eequ)/2d0*
     \(1d0-4d0*ABS(eequ)*sw2)*sigint
c
            sigmaij(2,-2)=sigmaij(2,-2)
     \+sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint
            sigmaij(3,-3)=sigmaij(3,-3)
     \+sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint
            sigmaij(5,-5)=sigmaij(5,-5)
     \+sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint
            sigmaij(-2,2)=sigmaij(-2,2)
     \+sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint
            sigmaij(-3,3)=sigmaij(-3,3)
     \+sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint
            sigmaij(-5,5)=sigmaij(-5,5)
     \+sigma0*sqrt(sw2/cw2)*ABS(eeqd)/2d0*
     \(1d0-4d0*ABS(eeqd)*sw2)*sigint

      endif



c%%%%%%%%%%%%%%%%%%%
      call idate(today)   ! today(1)=day, (2)=month, (3)=year
      call itime(now)     ! now(1)=hour, (2)=minute, (3)=second
      write(*,*) "("
      write ( *, 1000 )  today(1), today(2), today(3), now
 1000 format ( ' ( PROGRAM DYqT (v1.0)     Date: ', i2.2, '/', i2.2, '/'
     /, i4.4, '  Time: ',         i2.2, ':', i2.2, ':', i2.2 )
      write(*,*) "("
      write(*,*) "( G. Bozzi, S. Catani, G. Ferrera, D. de Florian, M. G
     /razzini"
      write(*,*) "( Phys. Lett. B696 (2011) 207 [arXiv:1007.2351 [hep-ph
     /]],"
      write(*,*) "( Nucl. Phys. B815 (2009) 174 [arXiv:0812.2862 [hep-ph
     /]];"
      write(*,*) "("

      if(flag5.eq.2)  write(*,*) "( W^+ and W^- Production"
      if(flag5.eq.21) write(*,*) "( W^+ Production"
      if(flag5.eq.22) write(*,*) "( W^- Production"
      if(flag5.eq.1)  write(*,*) "( Photon Production"
      if(flag5.eq.3) write(*,*)  "( Z Production"
      if(flag5.eq.5) write(*,*)  "( Z/Photon Production"
      write(*,*)  "( "

      if((flag5.eq.21.or.flag5.eq.22.or.flag5.eq.2).and.fnwa.eq.1) then
      write(*,*) "( BR(W)=",brw
      endif

      if(flag5.eq.3.and.fnwa.eq.1) write(*,*) "( BR(Z)=",brz
      write(*,*) '( alphas(mZ^2)='
     /,dreal(alphas(dcmplx(91.1876d0**2)))*4*pi,
     /'   alphas(1 GeV^2)=',dreal(alphas(dcmplx(1.0d0**2)))*4*pi
      write(*,*) "("
      write(12,*) "("
      write (12, 1000 )  today(1), today(2), today(3), now
      write(12,*) "("

      if(flag5.eq.2)  write(12,*) "( W^+ and W^- Production"
      if(flag5.eq.21) write(12,*) "( W^+ Production"
      if(flag5.eq.22) write(12,*) "( W^- Production"
      if(flag5.eq.1)  write(12,*) "( Photon Production"
      if(flag5.eq.3) write(12,*)  "( Z Production"
      if(flag5.eq.5) write(12,*)  "( Z/Photon Production"
      write(*,*)  "( "

      if((flag5.eq.21.or.flag5.eq.22.or.flag5.eq.2).and.fnwa.eq.1) then
      write(12,*) "( BR(W)=",brw
      endif

      if(flag5.eq.3.and.fnwa.eq.1) write(12,*) "( BR(Z)=",brz

      write(12,*) '( alphas(mZ^2)='
     /,dreal(alphas(dcmplx(91.1876d0**2)))*4*pi,
     /'   alphas(1 GeV^2)=',dreal(alphas(dcmplx(1.0d0**2)))*4*pi
      write(12,*) "("
c%%%%%%%%%%%%%%%%%%%


c.....Here starts the integration: you can choose 
c.....the parameters v,phi,min,max
c.....To integrate on complex plane set phi=pi/8,max=10
c........and set the Sudakov without the b-cutoff
c.....To integrate on real axis set phi=0,max=1
c........and set the Sudakov with the b-cutoff
c     flagrealcomplex=0(1) real(complex)

      flagrealcomplex=0
      flagrealcomplex=1

      if (flagrealcomplex.eq.0) then
C     real axis
         v=1d0
         phi=0d0
         min=1d-5
         max=3d0
      else
c  complex plane
         v=1d0
!         phi=pi/9
!         min=1d-5
!         max=10d0
         phi=pi/4
!         min=1d-5
!         max=3d0
         min=1d-10
         max=2.8d0/(2d0*qt**2/300**2+1d0)+0.2d0
      endif

c%%%%%%%%%%%%%%%%%%%%%%%
          write(*,*)"( input file:"
          write(*,*)"( ic=",ic
          write(*,*)"( V=",flag5
          write(*,*)"( NWA=",fnwa
          write(*,*)"( removeBR=",brflag
          write(*,*)"( q=",q
          write(*,*)"( order=",flag1
          write(*,*)"( filename=",runstring
          write(*,*)"( PDF,loops=",isetproton,nloop
          write(*,*)"( LHAPDF name,mem=",name2,mem
          write(*,*)"( sqrt(s)=",sroot
          write(*,*)"( muR,muF,muQ=",mur,muf,muq
          write(*,*)"( NP g-par=",g
          write(*,*)"( qtmin,qtmax,qtbin=",qtmin,qtmax1,qtbin1
          write(*,*)"( Vegas: iters., LO calls, NLO calls",iter,locall
     /,nlocall
          write(*,*)"( "
         if(flag1.ne.0) then
          write(*,*)"(--------------------------------------------------
     /------------------------"
          write(*,*)"(","      qt  ","         resum  ","         asym "
     /,"         fixed  ","        matched   "!," bin"
          write(*,*)"(--------------------------------------------------
     /------------------------"
          write(*,*)"("
          else
          write(*,*) "(--------------------------------"
          write(*,*) "(","      qt  ","         resum  "!," bin "
          write(*,*) "(--------------------------------"
          write(*,*) "("
         endif

          write(12,*)"( input file:"
          write(12,*)"( ic=",ic
          write(12,*)"( V=",flag5
          write(12,*)"( NWA=",fnwa
          write(12,*)"( removeBR=",brflag
          write(12,*)"( q=",q
          write(12,*)"( order=",flag1
          write(12,*)"( filename=",runstring
          write(12,*)"( PDF,loops=",isetproton,nloop
          write(12,*)"( LHAPDF name,mem=",name2,mem
          write(12,*)"( sqrt(s)=",sroot
          write(12,*)"( muR,muF,muQ=",mur,muf,muq
          write(12,*)"( NP g-par=",g
          write(12,*)"( qtmin,qtmax,qtbin=",qtmin,qtmax1,qtbin1
          write(12,*)"( Vegas: iters., LO calls, NLO calls",iter,locall
     /,nlocall
          write(12,*)"( "
         if(flag1.ne.0) then
          write(12,*)"(-------------------------------------------------
     /-------------------------"
          write(12,*)"(","      qt  ","         resum  ","         asym"
     /,"          fixed  ","        matched   "!," bin"
          write(12,*)"(-------------------------------------------------
     /-------------------------"
          write(12,*)"("
          else
          write(12,*) "(--------------------------------"
          write(12,*) "(","      qt  ","         resum  "!," bin "
          write(12,*) "(--------------------------------"
          write(12,*) "("
         endif
c%%%%%%%%%%%%%%%%%%%%%%%

      write(*,*)'( Wait for PDF fit...'

c.....this is one-time calculation of luminosities for LL result
      if (flag1.eq.0) then
         call evolfast (dcmplx(muf2),qqb1,qg1,gg1)
         LLqqb=qqb1
      endif

         qt=qtmin-qtbin1/2
         qtimax=(qtmax1-qtmin)/qtbin1
!     /         +(qtmax2-qtmax1)/qtbin2+(qtmax3-qtmax2)/qtbin3  !
!     /         +(qtmax4-qtmax3)/qtbin4                         !
         do qti=1,qtimax,1                            
!            if (qt.lt.qtmax1-qtbin1/2d0) then       !
               qtbin=qtbin1
!            elseif (qt.lt.qtmax1) then              !
!               qt=qt+qtbin1/2d0-qtbin2/2d0          !
!               qtbin=qtbin2                         !
!            elseif (qt.lt.qtmax2-qtbin2/2d0) then   !
!               qtbin=qtbin2                         ! 
!            elseif (qt.lt.qtmax2) then              !
!               qt=qt+qtbin2/2d0-qtbin3/2d0          ! 
!               qtbin=qtbin3                         !  
!            elseif (qt.lt.qtmax3-qtbin3/2d0) then   !    
!               qtbin=qtbin3                         ! 
!            elseif (qt.lt.qtmax3) then              !     
!               qt=qt+qtbin3/2d0-qtbin4/2d0          !      
!               qtbin=qtbin4                         !    
!            elseif (qti.le.qtmax4) then             !       
!               qtbin=qtbin4                         !          
!            endif                                   !
            qt=qt+qtbin

            errest=0d0

         qtbis=qt
         qt2=qt**2

c.....Initializing values and jacobian
         tmp1=0d0
         tmp2=0d0
         jac=cos(phi)+II*sin(phi)         
c....."upper-real" integral
         j=1
c         tmp1=adpint(fu,min,max,1d-10,1d-5,errest,ifail)    
         tmp1=adzint(fu,min,max,1d-10,1d-5,errest,ifail,0,0)    
c....."upper-complex" integral
         j=2
c         tmp2=adpint(fu,min,max,1d-10,1d-5,errest,ifail)
         tmp2=adzint(fu,min,max,1d-10,1d-5,errest,ifail,0,0)
         ru=tmp1+II*tmp2                
c.....Reinitializing values and jacobian
         tmp1=0d0
         tmp2=0d0
         jac=cos(phi)-II*sin(phi)         
c....."lower-real" integral
         j=1
c         tmp1=adpint(fd,min,max,1d-10,1d-5,errest,ifail)
         tmp1=adzint(fd,min,max,1d-10,1d-5,errest,ifail,0,0)
c....."lower-complex" integral
         j=2
c         tmp2=adpint(fd,min,max,1d-10,1d-5,errest,ifail)
         tmp2=adzint(fd,min,max,1d-10,1d-5,errest,ifail,0,0)
         rd=tmp1+II*tmp2   
c.....Now this is (d_sigma/d_qt) 
         rr=(ru+rd)
         allres=2/shad*qt*dreal(rr)/4d0

c.....If flag1=0 no matching

         if(flag1.ne.0) then
         call subtraction (dcmplx(muf2),subqqb,subqg,subgg)
            expres=(subqqb)*2/shad/qt 
         call fodyqt(ris) 
         write(*,90) qt,allres,expres,2*qt*ris
     /,(allres+2*qt*ris-expres)!,qtbin

         write(12,90) qt,allres,expres,2*qt*ris
     /,(allres+2*qt*ris-expres)!,qtbin
         else
            write(*,90) qt,allres!,qtbin
            write(12,90) qt,allres!,qtbin
         endif

      enddo
      close (12)

  90  format(8(1PE15.7))

      end
      
c.....Function to be integrated in the upper complex b plane
      function fu(t)
      implicit none
      integer j,flag1,iord,nloop
      real *8 t,fu
      complex *16 qqb,qg,gg,LLqqb
      complex *16 tmp,b,jac,h1,S
      complex*16 final
      common/order/iord
      common/nloop/nloop
      common/jac/jac
      common/j/j
      common/flag1/flag1
      common/LLqqb/LLqqb

      real *8 a_param,b0p
      common/a_param/a_param,b0p

      external h1,S
      include 'scales.h'
      include 'const.h'
      if (flag1.eq.0) then
         b=jac*t
         tmp=jac*b*h1(qt*b)*S(b)*LLqqb
         if(j.eq.1) then
            fu=dreal(tmp)
         elseif(j.eq.2) then
            fu=dimag(tmp)
         endif
      else
         b=jac*t
         final=(b0**2)/(b**2)       ! b0 and not b0p
         call evolfast(final,qqb,qg,gg)
         tmp=jac*b*h1(qt*b)*S(b)*qqb
         if(j.eq.1) then
            fu=dreal(tmp)
         elseif(j.eq.2) then
            fu=dimag(tmp)
         endif
      endif
      return
      end
      
c.....Function to be integrated in the lower complex b plane
      function fd(t)
      implicit none
      integer j,flag1,iord,nloop
      real *8 t,fd
      complex *16 qqb,qg,gg,LLqqb
      complex *16 tmp,b,jac,h2,S
      complex*16 final
      common/order/iord
      common/nloop/nloop
      common/jac/jac
      common/j/j
      common/flag1/flag1
      common/LLqqb/LLqqb

      real *8 a_param,b0p
      common/a_param/a_param,b0p

      external h2,S
      include 'scales.h'
      include 'const.h'
      if (flag1.eq.0) then
         b=jac*t
         tmp=jac*b*h2(qt*b)*S(b)*LLqqb
         if(j.eq.1) then
            fd=dreal(tmp)
         elseif(j.eq.2) then
            fd=dimag(tmp)
         endif
      else
         b=jac*t
         final=(b0**2)/(b**2)     !  b0 and not b0p
         call evolfast(final,qqb,qg,gg)
         tmp=jac*b*h2(qt*b)*S(b)*qqb
         if(j.eq.1) then
            fd=dreal(tmp)
         elseif(j.eq.2) then
            fd=dimag(tmp)
         endif
      endif
      return
      end

c.....Sudakov form factor
      function S(b)
      implicit none
      integer flag1,flagrealcomplex,imod
      complex *16 S,f0,f1,f2,b,bstar,blim,blog
      real*8 a_param,b0p,as,g
      common/asNEW/as
      common/flag1/flag1
      common/flagrealcomplex/flagrealcomplex
      common/modified/imod
      common/a_param/a_param,b0p
      common/NP/g
      include 'scales.h'
      include 'const.h'
      blim=(1/q)*exp(1/(2*as*beta0))

      bstar=b
c.....choose bstar (b) for real axis (complex plane) integration
CC      if (flagrealcomplex.eq.0) bstar=b/sqrt(1+(b**2)/(blim**2))

      if (imod.eq.1) blog=log( (q*bstar/b0p)**2+1)    !modified sudakov
      if (imod.eq.0) blog= log( (q*bstar/b0p)**2 )    !normal sudakov

      if (flag1.eq.0) then
         S=exp(blog*f0(beta0*as*blog))
      elseif (flag1.eq.1) then 
         S=exp(blog*f0(beta0*as*blog) +
     /                           f1(beta0*as*blog))
      elseif (flag1.eq.2) then 
         S=exp(blog*f0(beta0*as*blog) 
     /                      +     f1(beta0*as*blog) 
     /                      +  as*f2(beta0*as*blog)
     / )
      endif
      S=S*exp(-g*b**2)
      return
      end
      
      
c.....Soft-gluon-Resummation of LL
      function f0(y)
      implicit none
      complex *16 f0,y
      include 'const.h'
      include 'scales.h'
      f0=(A1q/beta0)*(y+log(1-y))/(y)
      return
      end
      
c.....Soft-gluon-Resummation of NLL
c.....Now we have mu_r dependence!
      function f1(y)
      implicit none
      complex *16 f1,y
      include 'const.h'
      include 'scales.h'
      real *8 a_param,b0p
      common/a_param/a_param,b0p

      f1=((A1q*beta1)/(beta0**3))*((1d0/2)*log(1-y)*log(1-y) +
     \     (y)/(1-y)+log(1-y)/(1-y)) -
     \     (A2q/(beta0**2))*(log(1-y)+(y)/(1-y)) + 
     \     (B1q/beta0)*log(1-y) +
     \     (A1q/beta0)*(y/(1-y)+log(1-y))*log(q2/mur2)


c    a dependence
      
      f1=f1-2*log(a_param)*A1q/beta0*y/(1-y)
      return
      end

c.....Soft-gluon-Resummation of NNLL
c.....Now we have mu_r dependence!
      function f2(y)
      implicit none
      complex *16 f2,y
      include 'const.h'
      include 'scales.h'
      real *8 a_param,b0p
      common/a_param/a_param,b0p

      f2=((A2q*beta1)/(beta0**3))*((y/2)*((3*y-2d0)/(1-y)**2)-
     \     ((1-2*y)*log(1-y)/(1-y)/(1-y))) - 
     \     (B2q/beta0)*((y)/(1-y))+
     \     (B1q*beta1/beta0**2)*((y)/(1-y)+log(1-y)/(1-y))-
     \     (A3q/2/beta0**2)*(y)*(y)/(1-y)/(1-y) +
     \     A1q*((beta1**2/2/beta0**4)*(1-2*y)/(1-y)
     \     /(1-y)*log(1-y)*log(1-y) +
     \     log(1-y)*((beta0*beta2-beta1**2)/(beta0**4)+
     \     beta1**2/beta0**4/(1-y)) +
c    \     beta1**4/beta0**4/(1-y)) +
     \     (y)/(2*beta0**4*(1-y)*(1-y))*
     \     (beta0*beta2*(2d0-3*y)+beta1**2*y)) -
     \     (A1q/2)*(y)*(y)/(1-y)/(1-y)*log(q2/mur2)*log(q2/mur2) +
     \     log(q2/mur2)*(B1q*y/(1-y)+A2q/beta0*y*y/(1-y)/(1-y)+
     \     A1q*beta1/beta0**2*(y/(1-y)+(1-2*y)/(1-y)/(1-y)*log(1-y))) 
     \     +2d0*C1qqn*((y)/(1-y))

c    a dependence (now without constant term)

      f2=f2+2*A1q*y*(y-2)/(1-y)**2*log(a_param)**2-log(a_param)
     \ *(2*B1q*y/(1-y)+2*y/beta0*A2q/(1-y)**2
     \ -2*A1q*beta1/beta0**2*y*log(1-y)/(1-y)**2)
!
     \ + A1q*log(a_param)*log(q2/mur2)*y*2d0/(1-y)**2


      return
      end
      
    
