*****************************************************************************
c.....written by giuseppe bozzi
c.....This program computes dsigma/dqT^2 at NLO accuracy
c.....for (gamma,W,Z)+jet production at hadron colliders 
c.....Based on: Gonsalves, Pawlowski, Wai - Phys.Rev.D40 (1989)
c*****************************************************************************

      subroutine fodyqt(ris)
      implicit real *8 (a-h,o-z)
      integer ih1,ih2,ic,isetproton,nloop,j,prodflag,nf,ord,nc
      integer qtpts,iter,locall,nlocall,iord,flagch
      common/flagch/flagch
      common/nf/nf
      common/scales2/xmur,xmuf,xmur2,xmuf2
      common/order/iord
      common/isetproton/isetproton
      common/pdf/ih1,ih2
      common/nloop/nloop
      common/para/s,ss
      common/yv/yv
      common/internal/qt,q,q2
      common/quarks/eq(5),alq(5),arq(5),ckm(6,6),delta(5,5),tau3(5,5)
      common/couplings/xw,cw,sw,alpha0
      common/const2/pi,cf,ca,tr,xnc
      common/gevpb/gevpb
      common/prodflag/prodflag
      common/vegint/iter,locall,nlocall
      common/ckm/vud,vus,vub,vcd,vcs,vcb,vtd,vts,vtb
      common/cdyqt/amv,y1,y2,qtbis,gf,ppi,ssroot,sw2,aem,ic

c.....definition of constants and couplings
c.....quarks are ordered according to mass:
c.....1,2,3,4,5=u,d,s,c,b,t
      ca=3d0
      xnc=3d0
      cf=4/3d0
      tr=nf/2d0
      pi=ppi
      alpha0=aem
      xw=sw2
      sw=dsqrt(xw)                       ! sin_w ! 
      cw=dsqrt(1-xw)                     ! cos_w !
      equ=2/3d0                          ! up-quarks electric charge !
      eqd=-1/3d0                         ! down-quarks electric charge !
      eq(1)=equ                 
      eq(2)=eqd                 
      eq(3)=eqd                 
      eq(4)=equ                 
      eq(5)=eqd                 
c.....definition of 'generalized' ckm matrix:
c.....    (uu ud us uc ub ut)
c.....    (du dd ds dc db dt)
c.....ckm=(su sd ss sc sb st)
c.....    (cu cd cs cc cb ct)
c.....    (bu bd bs bc bb bt)
c.....    (tu td ts tc tb tt)
      do i=1,6
         do j=1,6
            ckm(i,j)=0d0
         enddo
      enddo
      ckm(1,2)=vud
      ckm(1,3)=vus
      ckm(1,5)=vub
      ckm(2,1)=vud
      ckm(2,4)=vcd
      ckm(2,6)=vtd
      ckm(3,1)=vus
      ckm(3,4)=vcs
      ckm(3,6)=vts
      ckm(4,2)=vcd
      ckm(4,3)=vcs
      ckm(4,5)=vcb
      ckm(5,1)=vub
      ckm(5,4)=vcb
      ckm(5,6)=vtb
      ckm(6,2)=vtd
      ckm(6,3)=vts
      ckm(6,5)=vtb
c.....definition of 'delta' matrix
      do i=1,nf
         do j=1,nf
            delta(i,j)=0d0
         enddo
      enddo
      do i=1,nf
         delta(i,i)=1d0
      enddo
c.....definition of tau3's Pauli matrix
      do i=1,nf
         do j=1,nf
            tau3(i,j)=0d0
         enddo
      enddo
      tau3(1,1)=1d0
      tau3(2,2)=-1d0
      tau3(3,3)=-1d0
      tau3(4,4)=1d0
      tau3(5,5)=-1d0


c.....read the input file
      if(ic.eq.1) then
         ih1=1                      !beam 1 @ LHC!
         ih2=1                      !beam 2 @ LHC!
      elseif(ic.eq.-1) then
         ih1=1                      !beam 1 @ Tevatron!
         ih2=-1                     !beam 2 @ Tevatron!
      endif
      ss=ssroot
      s=ss**2
      q=amv
      q2=q**2
      ord=iord
      qt=qtbis
      call qtdy(ris,error,chi2a,y1,y2,ord)
      return
      end

c******************************************************
c.....this subroutine returns the value of dsigma/dqt^2
c******************************************************

      subroutine qtdy(ris,error,chi2a,y1,y2,ord)
      implicit real *8 (a-h,o-z)
      integer nf,ih1,ih2,ic,isetproton,prodflag
      integer ord,iord,iwseed,ndim,nloop,n3,ncl3
      integer iter,locall,nlocall
      external xdelta,sing
      common/asNEW/as
      common/asp/asp
      common/nf/nf
      common/nloop/nloop
      common/scales2/xmur,xmuf,xmur2,xmuf2
      common/rapidities/yy1,yy2
      common/yv/yv
      common/isetproton/isetproton
      common/pdf/ih1,ih2
      common/order/iord
      common/para/s,ss
      common/tm/tm
      common/internal/qt,q,q2
      common/const2/pi,cf,ca,tr,xnc
      common/mand/sh,th,uh,s2
      common/gevpb/gevpb
      common/prodflag/prodflag
      integer flagch
      common/flagch/flagch
      common/vegint/iter,locall,nlocall
      common/couplings/xw,cw,sw,alpha0
!
      tm=dsqrt(q2+qt**2)

c.....y2 not greater than y1
      if(y1.gt.y2) then
         ris=0d0
         return
      endif
      
c.....kinematical limits on qt
      z=q2/s
      xr=(1-z)**2-4*z*(qt/q)**2
      if(xr.lt.0)then
         ris=0d0
         return
      endif

c.....kinematical limits on y
      tmpx=(q2+s)/ss/tm
      ymax=dlog((tmpx+dsqrt(tmpx**2-4))/2)
      if(y1.gt.ymax.or.y2.lt.(-ymax)) then
         ris=0
         return
      elseif(y1.lt.-ymax.and.y2.gt.ymax) then
         yy1=-ymax
         yy2=ymax     
      elseif(y1.lt.-ymax.and.y2.gt.-ymax.and.y2.lt.ymax) then
         yy1=-ymax
         yy2=y2
      elseif(y1.gt.-ymax.and.y1.lt.ymax.and.y2.gt.ymax) then
         yy1=y1
         yy2=ymax
      elseif(y1.gt.-ymax.and.y1.lt.ymax.and.y2.lt.ymax) then
         yy1=y1
         yy2=y2
      endif
      
c.....compute as at order=nloop
      asp=as*pi

      if(qt/q.le.0.1d0) then
      ncl3=2*locall
      n3=2*iter
      else
      ncl3=locall
      n3=iter
      endif


c.....integration of delta(s2) terms
      ndim=2
!      n3=iter
!      ncl3=locall
      iwseed=1
      if (flagch.eq.3.or.flagch.eq.12) then
      rdelta=0d0
      error=0d0
      else
      call xinteg(xdelta,ndim,n3,ncl3,av3,d3,chi2a)
      rdelta=av3
      error=d3
      endif

      if(qt/q.le.0.1d0) then
      ncl3=2*nlocall
      n3=2*iter
      else
      ncl3=nlocall
      n3=iter
      endif


c.....integration of non-delta(s2) terms
      if (iord.eq.1) then
         ndim=3
!         n3=iter
!         ncl3=nlocall
         call xinteg(sing,ndim,n3,ncl3,av3,d3,chi2a)
         rsing=(asp/2d0/pi)*av3
         error=error+d3
      elseif(iord.eq.0) then
        rsing=0d0
      endif

c.....final result     
      ris=rdelta+rsing
      return
      end

c************************************************************
c.....this function includes all the delta(s2) contributions
c************************************************************

      function xdelta(xx)
      implicit real *8 (a-h,o-z)
      real *8 xx(1:2)
      real *8 fh1(-5:5),fh2(-5:5)
      integer nf,ih1,ih2,ic,isetproton,nloop,ord,iord,prodflag
      integer flagch
      common/asp/asp
      common/nf/nf
      common/scales2/xmur,xmuf,xmur2,xmuf2
      common/rapidities/y1,y2
      common/yv/yv
      common/isetproton/isetproton
      common/pdf/ih1,ih2
      common/order/iord
      common/para/s,ss
      common/tm/tm
      common/fractions/x1,x2
      common/internal/qt,q,q2
      common/const2/pi,cf,ca,tr,xnc
      common/mand/sh,th,uh,s2
      common/quarks/eq(5),alq(5),arq(5),ckm(6,6),delta(5,5),tau3(5,5)
      common/couplings/xw,cw,sw,alpha0
      common/gevpb/gevpb
      common/prodflag/prodflag
      common/luminosities/xlumgg,xlumqg,xlumgq,xlumqgtr,xlumgqtr,
     /     xlumqqb,xlumqqbtr,xlumqqbdbb,xlumqqbdbc,xlumqqbdcc,
     /     xlumqqbLL,xlumqqbLR,xlumqq,xlumqqeaa,xlumqqead,
     /     xlumqqLL,xlumqqLR
      common/flagch/flagch
c...........................................................
c.....definition of quantitites for phase space integration
c...........................................................
      xjac=1d0                  !jacobian "initialization"
      t=xx(1)                   !variable introduced to later allow 0<x2<1
      
c.....vector boson rapidity
      if(y1.ne.y2) then
         yv=y1+(y2-y1)*xx(2)
         xjac=xjac*(y2-y1)
      else
         yv=y1
      endif

c.....lower limit of integration for x2
      x2min=(q2-ss*tm*dexp(-yv))/(ss*tm*dexp(yv)-s)
      if(x2min.gt.1d0.or.x2min.lt.0d0) then
         write(*,*) 'error in x2min'
         stop
      endif

c.....change of variable to allow integration over 0-1
c....."esp" introduced for better behavior at small qt
      esp=8d0
      x2=dexp((1-t**esp)*dlog(x2min))
      xjac=xjac*x2*dlog(x2min)*esp*t**(esp-1d0)

c.....definition of x1 in function of x2
      x1=(ss*tm*dexp(yv)*x2-q2)/(x2*s-ss*tm*dexp(-yv))

c.....imposing x1,x2<1
      tiny=1d-8
      if (x1.gt.1-tiny.or.x2.gt.1-tiny) then
         xdelta=0d0
         return
      endif
      
c.....definition of partonic mandelstam invariants and jacobian      
      sh=x1*x2*s
!CM      th=q2-ss*x2*tm*dexp(yv)
!CM      uh=q2-ss*x1*tm*dexp(-yv)
      uh=q2-ss*x2*tm*dexp(yv)
      th=q2-ss*x1*tm*dexp(-yv)
      xjj=abs(x2*s-ss*tm*dexp(-yv))

c.....common factor for all the contributions
      factor=gevpb*alpha0*asp*cf/sh

c.....compute parton luminosity
      call flavour
c.....leading order delta(s2) contributions
      xloqg=(factor/(xnc**2-1d0))*(Aqg0(sh,th,uh,q2)+Agq0(sh,th,uh,q2))
      xloqqb=(factor/xnc)*Aqqb0(sh,th,uh,q2)
            if (flagch.eq.0) then
               xlo=xloqg+xloqqb 
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
               xlo=xloqqb 
            elseif (flagch.eq.2) then
               xlo=xloqg 
            elseif (flagch.eq.3) then
               xlo=0d0
            endif
c.....next to leading order delta(s2) contributions
      if (iord.eq.1) then
         xnloqg=(factor/(xnc**2-1d0))*(
     /     Bqg1(sh,th,uh,q2)+Bgq1(sh,th,uh,q2)+
     /     Bqg2(sh,th,uh,q2)+Bgq2(sh,th,uh,q2)+
     /     Cqg1(sh,th,uh,q2)+Cgq1(sh,th,uh,q2)+
     /     Cqg2(sh,th,uh,q2)+Cgq2(sh,th,uh,q2)+
     /     Bqg3(sh,th,uh,q2)+Bgq3(sh,th,uh,q2)         
     /        )
         xnloqqb=(factor/xnc)*(
     /        Bqqb1(sh,th,uh,q2)+Bqqb2(sh,th,uh,q2)+
     /        Cqqb1(sh,th,uh,q2)+D0aa(sh,th,uh,q2)+
     /        Bqqb3(sh,th,uh,q2)
     /        )
            if (flagch.eq.0) then
               xnlo=xnloqg+xnloqqb 
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
               xnlo=xnloqqb 
            elseif (flagch.eq.2) then
               xnlo=xnloqg 
            elseif (flagch.eq.3) then
               xnlo=0d0 
            endif
      else
         xnlo=0d0
      endif

c.....total result
      xdelta=xlo+(asp/2/pi)*xnlo
      xdelta=-pi*xdelta*xjac/xjj
      return
      end

c**********************************************************
c.....this function includes all the singular contributions
c**********************************************************

      function sing(zz)
      implicit real *8 (a-h,o-z)
      real *8 zz(1:3),lb
      integer nf,ih1,ih2,ic,isetproton,nloop,ord,iord,prodflag
      common/asp/asp
      common/nf/nf
      common/scales2/xmur,xmuf,xmur2,xmuf2
      common/rapidities/y1,y2
      common/yv/yv
      common/isetproton/isetproton
      common/pdf/ih1,ih2
      common/order/iord
      common/para/s,ss
      common/tm/tm
      common/fractions/x1,x2
      common/internal/qt,q,q2
      common/const2/pi,cf,ca,tr,xnc
      common/mand/sh,th,uh,s2
      common/couplings/xw,cw,sw,alpha0
      common/gevpb/gevpb
      common/prodflag/prodflag
      integer flagch
      common/flagch/flagch
      common/luminosities/xlumgg,xlumqg,xlumgq,xlumqgtr,xlumgqtr,
     /     xlumqqb,xlumqqbtr,xlumqqbdbb,xlumqqbdbc,xlumqqbdcc,
     /     xlumqqbLL,xlumqqbLR,xlumqq,xlumqqeaa,xlumqqead,
     /     xlumqqLL,xlumqqLR

c.....definition of quantitites for phase space integration
c.....as defined in formula B.1 of Glosser-Schmidt paper
c     zz(1) <-> z1      
c     zz(2) <-> z2
c     zz(3) <-> yv

      xjacy=1d0                  !.....jacobian initialization

c.....vector boson rapidity
      if(y1.ne.y2) then
       yv=y1+(y2-y1)*zz(3)
       xjacy=xjacy*(y2-y1)
      elseif(y1.eq.y2) then
       yv=y1
      endif

c.....x1_0,x2_0 and modification by Massimiliano (dcut)
      x10=tm/ss*dexp(yv)
      x20=tm/ss*dexp(-yv)
      dcut=x10*(qt/tm)**2/(1-x10*(1-(qt/tm)**2))

c.....z1,z2,lambda_b and corresponding jacobian 
c.....(to allow integration from 0 to 1)
      xjac=1d0
      z1=x20+(1-dcut-x20)*zz(1)
      xjac=xjac*(1-dcut-x20)

      lb=qt**2/tm**2*z1/(1-z1)
      z2=x10*(1+lb)+(1-x10*(1+lb))*zz(2)
      xjac=xjac*(1-x10*(1+lb))     

c.....first term of the integrand for the plus prescription

c.....x1,x2
      x1=x10/z2*(1+lb)
      x2=x20/z1

c.....definition of partonic Mandelstam invariants
      sh=tm**2/z1/z2*(1+lb)
!CM      th=-tm**2/z1*(1-z1)*(1+lb)
!CM      uh=q2-tm**2/z2*(1+lb)      
      uh=-tm**2/z1*(1-z1)*(1+lb)
      th=q2-tm**2/z2*(1+lb)      
      s2=tm**2/z1/z2*(1-z2)*(1-z1)*(1+lb)

c.....imposing s2>0
      if(s2.lt.0) then
       write(*,*)'s2 < 0 !'
       stop
      endif

c.....phase space prefactor
      pre1=tm**2*(1+lb)/(z1*z2)**2

c.....imposing x1,x2 < 1
      tiny=1d-7
      if(x1.gt.1-tiny.or.x2.gt.1-tiny) then
         sing2=0d0
         sing1=0d0
         reg=0d0
         xspur=0d0
         xm10=0
      else

c.....compute parton luminosity
         call flavour

c.....common factor for all the contributions
         factor=gevpb*alpha0*asp*cf/sh

c.....regular contributions (non-delta and non-singular in s2)
         xrgg=(factor*(xnc/(xnc**2-1d0)**2))*(
     /        Cgg1(sh,th,uh,q2)+Cgg1x(sh,th,uh,q2)
     /        )
         xrqg=(factor/(xnc**2-1d0))*(
     /        Cqg3(sh,th,uh,q2,3)+Cgq3(sh,th,uh,q2,3)+
     /        Cqg3(sh,th,uh,q2,4)+Cgq3(sh,th,uh,q2,4)
     /        )
         xrqqb=(factor/xnc)*(
     /        Cqqb2(sh,th,uh,q2,4)+Cqqb2x(sh,th,uh,q2,4)+
     /        Daa(sh,th,uh,q2,4)+Daax(sh,th,uh,q2,4)+
     /        Dab(sh,th,uh,q2)+Dabx(sh,th,uh,q2)+
     /        Dbb(sh,th,uh,q2)+Dbbx(sh,th,uh,q2)+
     /        Dac(sh,th,uh,q2)+Dad(sh,th,uh,q2)+
     /        Dbc(sh,th,uh,q2)+Dbd(sh,th,uh,q2)+
     /        Dcc(sh,th,uh,q2)+Ddd(sh,th,uh,q2)+
     /        DcdLL(sh,th,uh,q2)+DcdLLx(sh,th,uh,q2)+
     /        DcdLR(sh,th,uh,q2)+DcdLRx(sh,th,uh,q2)
     /        )
         xrqq=(factor/xnc)*(1/2d0)*(
     /        Eac(sh,th,uh,q2)+Ebd(sh,th,uh,q2)+
     /        Ead(sh,th,uh,q2)+Ebc(sh,th,uh,q2)
     /        )
            if (flagch.eq.0) then
         xreg=xrgg+xrqg+xrqqb+xrqq
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
         xreg=xrqqb+xrqq
            elseif (flagch.eq.2) then
         xreg=xrqg
            elseif (flagch.eq.3) then
         xreg=xrgg
            endif

c.....(log(s2)/s2)_A+ contributions
         xsqg1=(factor/(xnc**2-1d0))*
     /        (Cqg3(sh,th,uh,q2,1)+Cgq3(sh,th,uh,q2,1))
         xsqqb1=(factor/xnc)*
     /        (Cqqb2(sh,th,uh,q2,1)+Cqqb2x(sh,th,uh,q2,1))
            if (flagch.eq.0) then
         xs1=xsqg1+xsqqb1
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
         xs1=xsqqb1
            elseif (flagch.eq.2) then
         xs1=xsqg1
            elseif (flagch.eq.3) then
          xs1=0d0
            endif

c.....(1/s2)_A+ contributions
         xsqg2=(factor/(xnc**2-1d0))*
     /        (Cqg3(sh,th,uh,q2,2)+Cgq3(sh,th,uh,q2,2))
         xsqqb2=(factor/xnc)*(
     /        Cqqb2(sh,th,uh,q2,2)+Cqqb2x(sh,th,uh,q2,2)+
     /        Daa(sh,th,uh,q2,2)+Daax(sh,th,uh,q2,2)
     /        )
            if (flagch.eq.0) then
         xs2=xsqg2+xsqqb2
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
         xs2=xsqqb2
            elseif (flagch.eq.2) then
         xs2=xsqg2
            elseif (flagch.eq.3) then
         xs2=0d0
            endif

      endif 
      
c.....subtraction of z2=1 term according to plus prescription     

c.....x1,x2
      x1=x10*(1+lb)
      x2=x20/z1    

c.....definition of partonic Mandelstam invariants
      sh=tm**2/z1*(1+lb)
!CM      th=-tm**2/z1*(1-z1)*(1+lb)
!CM      uh=q2-tm**2*(1+lb) 
      uh=-tm**2/z1*(1-z1)*(1+lb)
      th=q2-tm**2*(1+lb) 
      s2=0d0

c.....phase space prefactor
      pre10=tm**2*(1+lb)/(z1)**2

c.....imposing x1,x2 < 1
      if(x1.gt.1-tiny.or.x2.gt.1-tiny) then
         sing2=0d0
         sing1=0d0
         reg=0d0
         xspur=0d0
         xm10=0
      else

c.....compute parton luminosity
         call flavour
         
c.....common factor for all the contributions
         factor=gevpb*alpha0*asp*cf/sh

c.....(log(s2)/s2)_A+ contributions
         xsqg10=(factor/(xnc**2-1d0))*
     /        (Cqg3(sh,th,uh,q2,1)+Cgq3(sh,th,uh,q2,1))
         xsqqb10=(factor/xnc)*
     /        (Cqqb2(sh,th,uh,q2,1)+Cqqb2x(sh,th,uh,q2,1))
            if (flagch.eq.0) then
         xs10=xsqg10+xsqqb10
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
         xs10=xsqqb10
            elseif (flagch.eq.2) then
         xs10=xsqg10
            elseif (flagch.eq.3) then
         xs10=0d0
            endif

c.....(1/s2)_A+ contributions
         xsqg20=(factor/(xnc**2-1d0))*
     /        (Cqg3(sh,th,uh,q2,2)+Cgq3(sh,th,uh,q2,2))
         xsqqb20=(factor/xnc)*(
     /        Cqqb2(sh,th,uh,q2,2)+Cqqb2x(sh,th,uh,q2,2)+
     /        Daa(sh,th,uh,q2,2)+Daax(sh,th,uh,q2,2)
     /        )
            if (flagch.eq.0) then
         xs20=xsqg20+xsqqb20
            elseif (flagch.eq.1.or.flagch.eq.11.or.flagch.eq.12) then
         xs20=xsqqb20
            elseif (flagch.eq.2) then
         xs20=xsqg20
            elseif (flagch.eq.3) then
         xs20=0d0
            endif


c.....final result.................................................
c.....note: here I am passing from s2 to z2 distributions..........
c.....according to the formulas:...................................
c.....(1/s2)_A+ = (1/-t)*((1/(1-z2))_+ -1 
c.....                   + log(z2min/(1-z2min))*delta(1-z2))
c.....(log(s2/q2)/s2)_A+ = log(-t/q2)*(1/s2)_A+ - 1/t*(
c.....                   (log(1-z2)/(1-z2))_+ + log(z2/(1-z2))
c.....                   - log(z2)/(1-z2) - 1/2*delta(1-z2)*
c.....                   log^2((1-z2min)/z2min))
c.....as a consequence, there appear some "spurious" terms.........
c.....in nlo delta, 1/s2 and regular contributions.................     
c.....I do not include spurious delta terms: they cancel against...
c.....similar contributions in subroutine 'xdelta'.................
c.....provided that A=-th in Gonsalves' formulas...................
c.....I also add mismatch terms (xmism) coming from 0-z2min region.

         z2min=x10*(1+lb)   

         xmism=-0.5d0*xs10*(dlog(1-z2min))**2-
     /        (xs10*dlog(-th/q2)+xs20)*dlog(1-z2min)

         xmism=xmism*pre10*(1-dcut-x20)/(-th)
         
         sing1=dlog(1-z2)/(1-z2)*(pre1*xs1-pre10*xs10)*xjac/(-th)

         sing2=(pre1*(xs1*dlog(-th/q2)+xs2)-
     /        pre10*(xs10*dlog(-th/q2)+xs20))/(1-z2)*xjac/(-th)

         reg=pre1*(xreg+
     /        (xs1*(dlog(z2/(1-z2))-(dlog(z2))/(1-z2)-dlog(-th/q2))-
     /        xs2)/(-th))*xjac
      endif
      
      sing=(sing1+sing2+reg-xmism)/s
!      write(*,*) reg
      sing=pi*xjacy*sing

      return
      end

